@@ -9,14 +9,27 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import cookielib
+import locale
 import os
-import StringIO
-import urllib2
-import urlparse
-import xmlrpclib
+import sys
 
-import pycurl
+from io import BytesIO
+
+if hasattr(sys.version_info, "major") and sys.version_info.major >= 3:
+    # pylint: disable=F0401,E0611
+    from configparser import SafeConfigParser
+    from http.cookiejar import LoadError, LWPCookieJar, MozillaCookieJar
+    from urllib.parse import urlparse, parse_qsl
+    from xmlrpc.client import (
+        Binary, Fault, ProtocolError, ServerProxy, Transport)
+else:
+    from ConfigParser import SafeConfigParser
+    from cookielib import LoadError, LWPCookieJar, MozillaCookieJar
+    from urlparse import urlparse, parse_qsl
+    from xmlrpclib import (
+        Binary, Fault, ProtocolError, ServerProxy, Transport)
+
+import requests
 
 from bugzilla import __version__, log
 from bugzilla.bug import _Bug, _User
@@ -29,14 +42,20 @@ mimemagic = None
 
 
 def _detect_filetype(fname):
+    # pylint: disable=E1103
+    # E1103: Instance of 'bool' has no '%s' member
+    # pylint confuses mimemagic to be of type 'bool'
     global mimemagic
 
     if mimemagic is None:
         try:
+            # pylint: disable=F0401
+            # F0401: Unable to import 'magic' (import-error)
             import magic
-            mimemagic = magic.open(magic.MAGIC_MIME_TYPE)
+            mimemagic = magic.open(getattr(magic, "MAGIC_MIME_TYPE", 16))
             mimemagic.load()
-        except ImportError, e:
+        except ImportError:
+            e = sys.exc_info()[1]
             log.debug("Could not load python-magic: %s", e)
             mimemagic = False
     if mimemagic is False:
@@ -47,33 +66,20 @@ def _detect_filetype(fname):
 
     try:
         return mimemagic.file(fname)
-    except Exception, e:
+    except Exception:
+        e = sys.exc_info()[1]
         log.debug("Could not detect content_type: %s", e)
     return None
 
 
-def _decode_rfc2231_value(val):
-    # BUG WORKAROUND: decode_header doesn't work unless there's whitespace
-    # around the encoded string (see http://bugs.python.org/issue1079)
-    from email import utils
-    from email import header
-
-    # pylint: disable=W1401
-    # Anomolous backslash in string
-    val = utils.ecre.sub(' \g<0> ', val)
-    val = val.strip('"')
-    return ''.join(f[0].decode(f[1] or 'us-ascii')
-                   for f in header.decode_header(val))
-
-
 def _build_cookiejar(cookiefile):
-    cj = cookielib.MozillaCookieJar(cookiefile)
+    cj = MozillaCookieJar(cookiefile)
     if cookiefile is None:
         return cj
     if not os.path.exists(cookiefile):
         # Make sure a new file has correct permissions
         open(cookiefile, 'a').close()
-        os.chmod(cookiefile, 0600)
+        os.chmod(cookiefile, 0o600)
         cj.save()
         return cj
 
@@ -82,90 +88,107 @@ def _build_cookiejar(cookiefile):
     try:
         cj.load()
         return cj
-    except cookielib.LoadError:
+    except LoadError:
         pass
 
     try:
-        cj = cookielib.LWPCookieJar(cookiefile)
+        cj = LWPCookieJar(cookiefile)
         cj.load()
-    except cookielib.LoadError:
+    except LoadError:
         raise BugzillaError("cookiefile=%s not in LWP or Mozilla format" %
                             cookiefile)
 
-    retcj = cookielib.MozillaCookieJar(cookiefile)
+    retcj = MozillaCookieJar(cookiefile)
     for cookie in cj:
         retcj.set_cookie(cookie)
     retcj.save()
     return retcj
 
 
-class _CURLTransport(xmlrpclib.Transport):
-    def __init__(self, url, cookiejar,
+class RequestsTransport(Transport):
+    user_agent = 'Python/Bugzilla'
+
+    def __init__(self, url, cookiejar=None,
                  sslverify=True, sslcafile=None, debug=0):
-        if hasattr(xmlrpclib.Transport, "__init__"):
-            xmlrpclib.Transport.__init__(self, use_datetime=False)
+        # pylint: disable=W0231
+        # pylint does not handle multiple import of Transport well
+        if hasattr(Transport, "__init__"):
+            Transport.__init__(self, use_datetime=False)
 
         self.verbose = debug
+        self._cookiejar = cookiejar
 
         # transport constructor needs full url too, as xmlrpc does not pass
         # scheme to request
-        self.scheme = urlparse.urlparse(url)[0]
+        self.scheme = urlparse(url)[0]
         if self.scheme not in ["http", "https"]:
             raise Exception("Invalid URL scheme: %s (%s)" % (self.scheme, url))
 
-        self.c = pycurl.Curl()
-        self.c.setopt(pycurl.POST, 1)
-        self.c.setopt(pycurl.CONNECTTIMEOUT, 30)
-        self.c.setopt(pycurl.HTTPHEADER, [
-            "Content-Type: text/xml",
-        ])
-        self.c.setopt(pycurl.VERBOSE, debug)
+        self.use_https = self.scheme == 'https'
 
-        self.set_cookiejar(cookiejar)
+        self.request_defaults = {
+            'cert': sslcafile if self.use_https else None,
+            'cookies': cookiejar if cookiejar else None,
+            'verify': sslverify,
+            'headers': {
+                'Content-Type': 'text/xml',
+                'User-Agent': self.user_agent,
+            }
+        }
 
-        # ssl settings
-        if self.scheme == "https":
-            # override curl built-in ca file setting
-            if sslcafile is not None:
-                self.c.setopt(pycurl.CAINFO, sslcafile)
+    def parse_response(self, response):
+        """ Parse XMLRPC response """
+        parser, unmarshaller = self.getparser()
+        parser.feed(response.text.encode('utf-8'))
+        parser.close()
+        return unmarshaller.close()
 
-            # disable ssl verification
-            if not sslverify:
-                self.c.setopt(pycurl.SSL_VERIFYPEER, 0)
-                self.c.setopt(pycurl.SSL_VERIFYHOST, 0)
-
-    def set_cookiejar(self, cj):
-        self.c.setopt(pycurl.COOKIEFILE, cj.filename or "")
-        self.c.setopt(pycurl.COOKIEJAR, cj.filename or "")
-
-    def get_cookies(self):
-        return self.c.getinfo(pycurl.INFO_COOKIELIST)
-
-    def open_helper(self, url, request_body):
-        self.c.setopt(pycurl.URL, url)
-        self.c.setopt(pycurl.POSTFIELDS, request_body)
-
-        b = StringIO.StringIO()
-        self.c.setopt(pycurl.WRITEFUNCTION, b.write)
+    def _request_helper(self, url, request_body):
+        """
+        A helper method to assist in making a request and provide a parsed
+        response.
+        """
+        response = None
         try:
-            self.c.perform()
-        except pycurl.error, e:
-            raise xmlrpclib.ProtocolError(url, e[0], e[1], None)
+            response = requests.post(
+                url, data=request_body, **self.request_defaults)
 
-        b.seek(0)
-        return b
+            # We expect utf-8 from the server
+            response.encoding = 'UTF-8'
+
+            # update/set any cookies
+            if self._cookiejar is not None:
+                for cookie in response.cookies:
+                    self._cookiejar.set_cookie(cookie)
+
+                if self._cookiejar.filename is not None:
+                    # Save is required only if we have a filename
+                    self._cookiejar.save()
+
+            response.raise_for_status()
+            return self.parse_response(response)
+        except requests.RequestException:
+            e = sys.exc_info()[1]
+            if not response:
+                raise
+            raise ProtocolError(
+                url, response.status_code, str(e), response.headers)
+        except Fault:
+            raise sys.exc_info()[1]
+        except Exception:
+            # pylint: disable=W0201
+            e = BugzillaError(str(sys.exc_info()[1]))
+            e.__traceback__ = sys.exc_info()[2]
+            raise e
 
     def request(self, host, handler, request_body, verbose=0):
         self.verbose = verbose
         url = "%s://%s%s" % (self.scheme, host, handler)
 
         # xmlrpclib fails to escape \r
-        request_body = request_body.replace('\r', '&#xd;')
-
-        stringio = self.open_helper(url, request_body)
-        return self.parse_response(stringio)
-
+        request_body = request_body.replace(b'\r', b'&#xd;')
 
+        return self._request_helper(url, request_body)
 
 
 class BugzillaError(Exception):
@@ -221,13 +244,16 @@ class BugzillaBase(object):
         be passed along to the Bugzilla.query() method.
         '''
         q = {}
-        (ignore, ignore, path,
-         ignore, query, ignore) = urlparse.urlparse(url)
 
-        if os.path.basename(path) not in ('buglist.cgi', 'query.cgi'):
+        # pylint: disable=unpacking-non-sequence
+        (ignore, ignore, path,
+         ignore, query, ignore) = urlparse(url)
+
+        base = os.path.basename(path)
+        if base not in ('buglist.cgi', 'query.cgi'):
             return {}
 
-        for (k, v) in urlparse.parse_qsl(query):
+        for (k, v) in parse_qsl(query):
             if k not in q:
                 q[k] = v
             elif isinstance(q[k], list):
@@ -236,6 +262,13 @@ class BugzillaBase(object):
                 oldv = q[k]
                 q[k] = [oldv, v]
 
+        # Handle saved searches
+        if base == "buglist.cgi" and "namedcmd" in q and "sharer_id" in q:
+            q = {
+                "sharer_id": q["sharer_id"],
+                "savedsearch": q["namedcmd"],
+            }
+
         return q
 
     @staticmethod
@@ -288,9 +321,8 @@ class BugzillaBase(object):
         self._components_details = {}
 
     def _get_user_agent(self):
-        ret = ('Python-urllib2/%s bugzilla.py/%s %s/%s' %
-               (urllib2.__version__, __version__,
-                str(self.__class__.__name__), self.version))
+        ret = ('Python-urllib bugzilla.py/%s %s/%s' %
+               (__version__, str(self.__class__.__name__), self.version))
         return ret
     user_agent = property(_get_user_agent)
 
@@ -368,11 +400,10 @@ class BugzillaBase(object):
         '''
         Read bugzillarc file(s) into memory.
         '''
-        import ConfigParser
         if not configpath:
             configpath = self.configpath
         configpath = [os.path.expanduser(p) for p in configpath]
-        c = ConfigParser.SafeConfigParser()
+        c = SafeConfigParser()
         r = c.read(configpath)
         if not r:
             return
@@ -380,16 +411,15 @@ class BugzillaBase(object):
         section = ""
         # Substring match - prefer the longest match found
         log.debug("Searching for config section matching %s", self.url)
-        for s in sorted(c.sections(),
-                        lambda a, b: cmp(len(a), len(b)) or cmp(a, b)):
+        for s in sorted(c.sections()):
             if s in self.url:
-                log.debug("Found matching section: %s" % s)
+                log.debug("Found matching section: %s", s)
                 section = s
         if not section:
             return
         for k, v in c.items(section):
             if k in ('user', 'password'):
-                log.debug("Setting '%s' from configfile" % k)
+                log.debug("Setting '%s' from configfile", k)
                 setattr(self, k, v)
 
     def connect(self, url=None):
@@ -406,10 +436,10 @@ class BugzillaBase(object):
             url = self.url
         url = self.fix_url(url)
 
-        self._transport = _CURLTransport(url, self._cookiejar,
-                                         sslverify=self._sslverify)
+        self._transport = RequestsTransport(
+            url, self._cookiejar, sslverify=self._sslverify)
         self._transport.user_agent = self.user_agent
-        self._proxy = xmlrpclib.ServerProxy(url, self._transport)
+        self._proxy = ServerProxy(url, self._transport)
 
 
         self.url = url
@@ -465,7 +495,7 @@ class BugzillaBase(object):
             self.logged_in = True
             log.info("login successful - dropping password from memory")
             self.password = ''
-        except xmlrpclib.Fault:
+        except Fault:
             r = False
 
         return r
@@ -496,15 +526,7 @@ class BugzillaBase(object):
         '''
         if force_refresh or self._bugfields is None:
             log.debug("Refreshing bugfields")
-            try:
-                self._bugfields = self._getbugfields()
-            except xmlrpclib.Fault, f:
-                if f.faultCode == 'Client':
-                    # okay, this instance doesn't have getbugfields. fine.
-                    self._bugfields = []
-                else:
-                    # something bad actually happened on the server. blow up.
-                    raise f
+            self._bugfields = self._getbugfields()
             self._bugfields.sort()
             log.debug("bugfields = %s", self._bugfields)
 
@@ -518,15 +540,22 @@ class BugzillaBase(object):
         Refresh a product's cached info
         Takes same arguments as _getproductinfo
         """
+        if self._products is None:
+            self._products = []
+
         for product in self._getproductinfo(**kwargs):
-            for current in self.products[:]:
+            added = False
+            for current in self._products[:]:
                 if (current.get("id", -1) != product.get("id", -2) and
                     current.get("name", -1) != product.get("name", -2)):
                     continue
 
-                self.products.remove(current)
-                self.products.append(product)
+                self._products.remove(current)
+                self._products.append(product)
+                added = True
                 break
+            if not added:
+                self._products.append(product)
 
     def getproducts(self, force_refresh=False, **kwargs):
         '''Get product data: names, descriptions, etc.
@@ -616,6 +645,7 @@ class BugzillaBase(object):
         '''
         data = data.copy()
         self._component_data_convert(data)
+        log.debug("Calling Component.create with: %s", data)
         return self._proxy.Component.create(data)
 
     def editcomponent(self, data):
@@ -627,6 +657,7 @@ class BugzillaBase(object):
         '''
         data = data.copy()
         self._component_data_convert(data, update=True)
+        log.debug("Calling Component.update with: %s", data)
         return self._proxy.Component.update(data)
 
 
@@ -664,6 +695,7 @@ class BugzillaBase(object):
 
         # The bugzilla4 name is Product.get(), but Bugzilla3 only had
         # Product.get_product, and bz4 kept an alias.
+        log.debug("Calling Product.get_products with: %s", kwargs)
         ret = self._proxy.Product.get_products(kwargs)
         return ret['products']
 
@@ -687,12 +719,22 @@ class BugzillaBase(object):
                                "fetching component details.")
 
         comps = None
-        for p in self.products:
-            if p["name"] != product:
-                continue
-            comps = p["components"]
+        if self._products is None:
+            self._products = []
 
-        if not comps:
+        def _find_comps():
+            for p in self._products:
+                if p["name"] != product:
+                    continue
+                return p.get("components", None)
+
+        comps = _find_comps()
+        if comps is None:
+            self.refresh_products(names=[product],
+                                  include_fields=["name", "id", "components"])
+            comps = _find_comps()
+
+        if comps is None:
             raise ValueError("Unknown product '%s'" % product)
 
         # Convert to old style dictionary to maintain back compat
@@ -753,7 +795,14 @@ class BugzillaBase(object):
 
         @simple: If True, don't ask for any large extra_fields.
         '''
-        idlist = [int(i) for i in idlist]
+        oldidlist = idlist
+        idlist = []
+        for i in oldidlist:
+            try:
+                idlist.append(int(i))
+            except ValueError:
+                # String aliases can be passed as well
+                idlist.append(i)
 
         getbugdata = {"ids": idlist}
         if permissive:
@@ -761,6 +810,7 @@ class BugzillaBase(object):
         if self.getbug_extra_fields and not simple:
             getbugdata["extra_fields"] = self.getbug_extra_fields
 
+        log.debug("Calling Bug.get_bugs with: %s", getbugdata)
         r = self._proxy.Bug.get_bugs(getbugdata)
 
         if self.bz_ver_major >= 4:
@@ -768,7 +818,21 @@ class BugzillaBase(object):
         else:
             bugdict = dict([(b['id'], b['internals']) for b in r['bugs']])
 
-        return [bugdict.get(i) for i in idlist]
+        ret = []
+        for i in idlist:
+            found = None
+            if i in bugdict:
+                found = bugdict[i]
+            else:
+                # Need to map an alias
+                for valdict in bugdict.values():
+                    if i in valdict.get("alias", []):
+                        found = valdict
+                        break
+
+            ret.append(found)
+
+        return ret
 
     def _getbug(self, objid, simple=False):
         '''Return a dict of full bug info for the given bug id'''
@@ -777,7 +841,6 @@ class BugzillaBase(object):
     def getbug(self, objid):
         '''Return a Bug object with the full complement of bug data
         already loaded.'''
-        log.debug("getbug(%s)" % str(objid))
         return _Bug(bugzilla=self, dict=self._getbug(objid))
 
     def getbugs(self, idlist):
@@ -841,7 +904,11 @@ class BugzillaBase(object):
                     target_milestone=None,
                     emailtype=None,
                     booleantype=None,
-                    include_fields=None):
+                    include_fields=None,
+                    quicksearch=None,
+                    savedsearch=None,
+                    savedsearch_sharer_id=None,
+                    sub_component=None):
         """
         Build a query string from passed arguments. Will handle
         query parameter differences between various bugzilla versions.
@@ -855,11 +922,6 @@ class BugzillaBase(object):
 
         Then pass the output to Bugzilla.query()
         """
-        # pylint: disable=W0221
-        # Argument number differs from overridden method
-        # Base defines it with *args, **kwargs, so we don't have to maintain
-        # the master argument list in 2 places
-
         ignore = include_fields
         ignore = emailtype
         ignore = booleantype
@@ -873,6 +935,11 @@ class BugzillaBase(object):
             ('devel_whiteboard', devel_whiteboard),
             ('alias', alias),
             ('boolean_query', boolean_query),
+            ('long_desc', long_desc),
+            ('quicksearch', quicksearch),
+            ('savedsearch', savedsearch),
+            ('sharer_id', savedsearch_sharer_id),
+            ('sub_component', sub_component),
         ]:
             if not val is None:
                 raise RuntimeError("'%s' search not supported by this "
@@ -882,7 +949,6 @@ class BugzillaBase(object):
             "product": self._listify(product),
             "component": self._listify(component),
             "version": version,
-            "long_desc": long_desc,
             "id": bug_id,
             "short_desc": short_desc,
             "bug_status": status,
@@ -903,15 +969,16 @@ class BugzillaBase(object):
         }
 
         # Strip out None elements in the dict
-        for key in query.keys():
-            if query[key] is None:
-                del(query[key])
+        for k, v in query.copy().items():
+            if v is None:
+                del(query[k])
         return query
 
     def _query(self, query):
         # This is kinda redundant now, but various scripts call
         # _query with their own assembled dictionaries, so don't
         # drop this lest we needlessly break those users
+        log.debug("Calling Bug.search with: %s", query)
         return self._proxy.Bug.search(query)
 
     def query(self, query):
@@ -921,7 +988,6 @@ class BugzillaBase(object):
         Also see the _query() method for details about the underlying
         implementation.
         '''
-        log.debug("Calling query with: %s", query)
         r = self._query(query)
         log.debug("Query returned %s bugs", len(r['bugs']))
         return [_Bug(bugzilla=self, dict=b) for b in r['bugs']]
@@ -981,6 +1047,7 @@ class BugzillaBase(object):
         tmp = updates.copy()
         tmp["ids"] = self._listify(ids)
 
+        log.debug("Calling Bug.update with: %s", tmp)
         return self._proxy.Bug.update(tmp)
 
     def update_flags(self, idlist, flags):
@@ -991,6 +1058,7 @@ class BugzillaBase(object):
          {"name": "devel_ack", "status": "-"}, ...]
         '''
         d = {"ids": self._listify(idlist), "updates": flags}
+        log.debug("Calling Flag.update with: %s", d)
         return self._proxy.Flag.update(d)
 
 
@@ -1041,7 +1109,8 @@ class BugzillaBase(object):
                      fixed_in=None,
                      qa_whiteboard=None,
                      devel_whiteboard=None,
-                     internal_whiteboard=None):
+                     internal_whiteboard=None,
+                     sub_component=None):
         # pylint: disable=W0221
         # Argument number differs from overridden method
         # Base defines it with *args, **kwargs, so we don't have to maintain
@@ -1054,6 +1123,7 @@ class BugzillaBase(object):
             ("devel_whiteboard", devel_whiteboard),
             ("qa_whiteboard", qa_whiteboard),
             ("internal_whiteboard", internal_whiteboard),
+            ("sub_component", sub_component),
         ]:
             if val is not None:
                 raise ValueError("bugzilla instance does not support "
@@ -1143,7 +1213,7 @@ class BugzillaBase(object):
     def attachfile(self, idlist, attachfile, description, **kwargs):
         '''
         Attach a file to the given bug IDs. Returns the ID of the attachment
-        or raises xmlrpclib.Fault if something goes wrong.
+        or raises XMLRPC Fault if something goes wrong.
 
         attachfile may be a filename (which will be opened) or a file-like
         object, which must provide a 'read' method. If it's not one of these,
@@ -1184,7 +1254,12 @@ class BugzillaBase(object):
             kwargs["file_name"] = kwargs.pop("filename")
 
         kwargs['summary'] = description
-        kwargs['data'] = xmlrpclib.Binary(f.read())
+
+        data = f.read()
+        if not isinstance(data, bytes):
+            data = data.encode(locale.getpreferredencoding())
+        kwargs['data'] = Binary(data)
+
         kwargs['ids'] = self._listify(idlist)
 
         if 'file_name' not in kwargs and hasattr(f, "name"):
@@ -1212,29 +1287,27 @@ class BugzillaBase(object):
     def openattachment(self, attachid):
         '''Get the contents of the attachment with the given attachment ID.
         Returns a file-like object.'''
+
+        def get_filename(headers):
+            import re
+
+            match = re.search(
+                r'^.*filename="?(.*)"$',
+                headers.get('content-disposition', '')
+            )
+
+            # default to attchid if no match was found
+            return match.group(1) if match else attachid
+
         att_uri = self._attachment_uri(attachid)
 
-        headers = {}
-        ret = StringIO.StringIO()
+        response = requests.get(att_uri, cookies=self._cookiejar, stream=True)
 
-        def headers_cb(buf):
-            if not ":" in buf:
-                return
-            name, val = buf.split(":", 1)
-            headers[name.lower()] = val
-
-        c = pycurl.Curl()
-        c.setopt(pycurl.URL, att_uri)
-        c.setopt(pycurl.WRITEFUNCTION, ret.write)
-        c.setopt(pycurl.HEADERFUNCTION, headers_cb)
-        c.setopt(pycurl.COOKIEFILE, self._cookiejar.filename or "")
-        c.perform()
-        c.close()
-
-        disp = headers['content-disposition'].split(';')
-        disp.pop(0)
-        parms = dict([p.strip().split("=", 1) for p in disp])
-        ret.name = _decode_rfc2231_value(parms['filename'])
+        ret = BytesIO()
+        for chunk in response.iter_content(chunk_size=1024):
+            if chunk:
+                ret.write(chunk)
+        ret.name = get_filename(response.headers)
 
         # Hooray, now we have a file-like object with .read() and .name
         ret.seek(0)
@@ -1275,6 +1348,8 @@ class BugzillaBase(object):
         comment_private=None,
         blocks=None,
         cc=None,
+        assigned_to=None,
+        keywords=None,
         depends_on=None,
         groups=None,
         op_sys=None,
@@ -1286,7 +1361,8 @@ class BugzillaBase(object):
         status=None,
         target_milestone=None,
         target_release=None,
-        url=None):
+        url=None,
+        sub_component=None):
 
         localdict = {}
         if blocks:
@@ -1297,6 +1373,8 @@ class BugzillaBase(object):
             localdict["depends_on"] = self._listify(depends_on)
         if groups:
             localdict["groups"] = self._listify(groups)
+        if keywords:
+            localdict["keywords"] = self._listify(keywords)
         if description:
             localdict["description"] = description
             if comment_private:
@@ -1309,7 +1387,8 @@ class BugzillaBase(object):
                 platform=platform, priority=priority, qa_contact=qa_contact,
                 resolution=resolution, severity=severity, status=status,
                 target_milestone=target_milestone,
-                target_release=target_release, url=url)
+                target_release=target_release, url=url,
+                assigned_to=assigned_to, sub_component=sub_component)
 
         ret.update(localdict)
         return ret
@@ -1337,8 +1416,6 @@ class BugzillaBase(object):
         else:
             data = kwargs
 
-        log.debug("bz.createbug(%s)", data)
-
         # If we're getting a call that uses an old fieldname, convert it to the
         # new fieldname instead.
         for newfield, oldfield in self.field_aliases:
@@ -1351,6 +1428,7 @@ class BugzillaBase(object):
         if "check_args" in data:
             del(data["check_args"])
 
+        log.debug("Calling Bug.create with: %s", data)
         rawbug = self._proxy.Bug.create(data)
         return _Bug(self, bug_id=rawbug["id"])
 
@@ -1366,7 +1444,7 @@ class BugzillaBase(object):
         :kwarg names: list of user names to return data on
         :kwarg match: list of patterns.  Returns users whose real name or
             login name match the pattern.
-        :raises xmlrpclib.Fault: Code 51: if a Bad Login Name was sent to the
+        :raises XMLRPC Fault: Code 51: if a Bad Login Name was sent to the
                 names array.
             Code 304: if the user was not authorized to see user they
                 requested.
@@ -1386,13 +1464,14 @@ class BugzillaBase(object):
             raise BugzillaError('_get() needs one of ids, '
                                 ' names, or match kwarg.')
 
+        log.debug("Calling User.get with: %s", params)
         return self._proxy.User.get(params)
 
     def getuser(self, username):
         '''Return a bugzilla User for the given username
 
         :arg username: The username used in bugzilla.
-        :raises xmlrpclib.Fault: Code 51 if the username does not exist
+        :raises XMLRPC Fault: Code 51 if the username does not exist
         :returns: User record for the username
         '''
         ret = self.getusers(username)
@@ -1434,7 +1513,7 @@ class BugzillaBase(object):
         :arg email: The email address to use in bugzilla
         :kwarg name: Real name to associate with the account
         :kwarg password: Password to set for the bugzilla account
-        :raises xmlrpclib.Fault: Code 501 if the username already exists
+        :raises XMLRPC Fault: Code 501 if the username already exists
             Code 500 if the email address isn't valid
             Code 502 if the password is too short
             Code 503 if the password is too long
@@ -1466,7 +1545,7 @@ class BugzillaBase(object):
             }
         }
 
-        log.debug("updating user permissions:\n%s", update)
+        log.debug("Call User.update with: %s", update)
         return self._proxy.User.update(update)
 
 
