@@ -27,11 +27,11 @@ class _Bug(object):
         self.bugzilla = bugzilla
 
         if 'dict' in kwargs and kwargs['dict']:
-            log.debug("Bug(%s)" % sorted(kwargs['dict'].keys()))
+            log.debug("Bug(%s)", sorted(kwargs['dict'].keys()))
             self._update_dict(kwargs['dict'])
 
         if 'bug_id' in kwargs:
-            log.debug("Bug(%i)" % kwargs['bug_id'])
+            log.debug("Bug(%i)", kwargs['bug_id'])
             setattr(self, 'id', kwargs['bug_id'])
 
         # Back compat for a previously handled param
@@ -48,10 +48,11 @@ class _Bug(object):
         '''Return a simple string representation of this bug
 
         This is available only for compatibility. Using 'str(bug)' and
-        'print bug' is not recommended because of potential encoding issues.
+        'print(bug)' is not recommended because of potential encoding issues.
         Please use unicode(bug) where possible.
         '''
-        return unicode(self).encode(locale.getpreferredencoding(), 'replace')
+        return self.__unicode__().encode(
+            locale.getpreferredencoding(), 'replace')
 
     def __unicode__(self):
         '''Return a simple unicode string representation of this bug'''
