@@ -54,7 +54,7 @@ class RHBugzilla(_parent):
     getbug_extra_fields = (
         _parent.getbug_extra_fields + [
             "attachments", "comments", "description",
-            "external_bugs", "flags",
+            "external_bugs", "flags", "sub_components",
         ]
     )
 
@@ -74,24 +74,36 @@ class RHBugzilla(_parent):
     # Bug update methods #
     ######################
 
-    def build_update(self, *args, **kwargs):
+    def build_update(self, **kwargs):
         adddict = {}
 
         def pop(key, destkey):
-            if key not in kwargs:
-                return
-
-            val = kwargs.pop(key)
+            val = kwargs.pop(key, None)
             if val is None:
                 return
             adddict[destkey] = val
 
+        def get_sub_component():
+            val = kwargs.pop("sub_component", None)
+            if val is None:
+                return
+
+            if type(val) is not dict:
+                component = self._listify(kwargs.get("component"))
+                if component is []:
+                    raise ValueError("component must be specified if "
+                        "specifying sub_component")
+                val = {component[0]: val}
+            adddict["sub_components"] = val
+
         pop("fixed_in", "cf_fixed_in")
         pop("qa_whiteboard", "cf_qa_whiteboard")
         pop("devel_whiteboard", "cf_devel_whiteboard")
         pop("internal_whiteboard", "cf_internal_whiteboard")
 
-        vals = _parent.build_update(self, *args, **kwargs)
+        get_sub_component()
+
+        vals = _parent.build_update(self, **kwargs)
         vals.update(adddict)
 
         return vals
@@ -125,8 +137,14 @@ class RHBugzilla(_parent):
                 query['include_fields'] = query['column_list']
                 del query['column_list']
 
+        include_aliases = (
+            ("component", "components"),
+            ("version", "versions"),
+            ("sub_components", "sub_component"),
+        )
+
         include_fields = query['include_fields']
-        for newname, oldname in self.field_aliases:
+        for newname, oldname in (self.field_aliases + include_aliases):
             if oldname in include_fields:
                 include_fields.remove(oldname)
                 if newname not in include_fields:
@@ -155,6 +173,17 @@ class RHBugzilla(_parent):
             bug['versions'] = type(val) is list and val or [val]
             bug['version'] = bug['versions'][0]
 
+        # sub_components isn't too friendly of a format, add a simpler
+        # sub_component value
+        if 'sub_components' in bug and 'sub_component' not in bug:
+            val = bug['sub_components']
+            bug['sub_component'] = ""
+            if type(val) is dict:
+                values = []
+                for vallist in val.values():
+                    values += vallist
+                bug['sub_component'] = " ".join(values)
+
         if not self.rhbz_back_compat:
             return
 
@@ -195,6 +224,23 @@ class RHBugzilla(_parent):
     def build_query(self, **kwargs):
         query = {}
 
+        def _add_key(paramname, keyname, listify=False):
+            val = kwargs.pop(paramname, None)
+            if val is None:
+                return
+            if listify:
+                val = self._listify(val)
+            query[keyname] = val
+
+        def add_longdesc():
+            val = kwargs.pop("long_desc", None)
+            if val is None:
+                return
+
+            query["query_format"] = "advanced"
+            query["longdesc"] = val
+            query["longdesc_type"] = "allwordssubstr"
+
         def add_email(key, count):
             if not key in kwargs:
                 return count
@@ -305,6 +351,13 @@ class RHBugzilla(_parent):
         chart_id = add_boolean("alias", "alias", chart_id)
         chart_id = add_boolean("boolean_query", None, chart_id)
 
+        add_longdesc()
+
+        _add_key("quicksearch", "quicksearch")
+        _add_key("savedsearch", "savedsearch")
+        _add_key("savedsearch_sharer_id", "sharer_id")
+        _add_key("sub_component", "sub_components", listify=True)
+
         newquery = _parent.build_query(self, **kwargs)
         query.update(newquery)
         self.pre_translation(query)
