@@ -9,12 +9,19 @@
 Unit tests that do permanent functional against a real bugzilla instances.
 '''
 
+from __future__ import print_function
+
 import datetime
 import os
 import random
 import sys
 import unittest
-import urllib2
+
+if hasattr(sys.version_info, "major") and sys.version_info.major >= 3:
+    # pylint: disable=F0401,E0611
+    from urllib.parse import urlparse
+else:
+    from urlparse import urlparse
 
 import bugzilla
 from bugzilla import Bugzilla
@@ -34,13 +41,13 @@ class BaseTest(unittest.TestCase):
 
     def _testBZClass(self):
         bz = Bugzilla(url=self.url, cookiefile=None)
-        self.assertTrue(isinstance(bz, self.bzclass))
+        self.assertTrue(bz.__class__ is self.bzclass)
 
     def _testCookie(self):
         cookiefile = cf
-        domain = urllib2.urlparse.urlparse(self.url)[1]
+        domain = urlparse(self.url)[1]
         if os.path.exists(cookiefile):
-            out = file(cookiefile).read(1024)
+            out = open(cookiefile).read(1024)
             if domain in out:
                 return
 
@@ -63,13 +70,13 @@ class RHPartnerTest(BaseTest):
         # Check a known account that likely won't ever go away
         ret = bool(bz.getuser("anaconda-maint-list@redhat.com").groupnames)
         if not ret:
-            print "\nNo admin privs, skipping %s" % funcname
+            print("\nNo admin privs, skipping %s" % funcname)
         return ret
 
     def _check_rh_privs(self, bz, funcname, quiet=False):
         noprivs = bool(bz.getbugs([184858]) == [None])
         if noprivs and not quiet:
-            print "\nNo RH privs, skipping %s" % funcname
+            print("\nNo RH privs, skipping %s" % funcname)
         return not noprivs
 
 
@@ -97,7 +104,7 @@ class RHPartnerTest(BaseTest):
 
         bugid = int(newout.splitlines()[2])
         bug = bz.getbug(bugid)
-        print "\nCreated bugid: %s" % bugid
+        print("\nCreated bugid: %s" % bugid)
 
         # Verify hasattr works
         self.assertTrue(hasattr(bug, "id"))
@@ -129,20 +136,23 @@ class RHPartnerTest(BaseTest):
         blocked = "461686,461687"
         dependson = "427301"
         comment = "Test bug from python-bugzilla test suite"
+        sub_component = "Command-line tools (RHEL6)"
         newout = tests.clicomm("bugzilla new "
-            "--product Fedora --component python-bugzilla --version rawhide "
+            "--product 'Red Hat Enterprise Linux 6' --version 6.0 "
+            "--component lvm2 --sub-component '%s' "
             "--summary \"%s\" "
             "--comment \"%s\" "
             "--url %s --severity Urgent --priority Low --os %s "
             "--arch ppc --cc %s --blocked %s --dependson %s "
             "--outputformat \"%%{bug_id}\"" %
-            (summary, comment, url, osval, cc, blocked, dependson), bz)
+            (sub_component, summary, comment, url,
+             osval, cc, blocked, dependson), bz)
 
         self.assertTrue(len(newout.splitlines()) == 3)
 
         bugid = int(newout.splitlines()[2])
         bug = bz.getbug(bugid)
-        print "\nCreated bugid: %s" % bugid
+        print("\nCreated bugid: %s" % bugid)
 
         self.assertEquals(bug.summary, summary)
         self.assertEquals(bug.bug_file_loc, url)
@@ -151,6 +161,7 @@ class RHPartnerTest(BaseTest):
         self.assertEquals(bug.depends_on, _split_int(dependson))
         self.assertTrue(all([e in bug.cc for e in cc.split(",")]))
         self.assertEquals(bug.longdescs[0]["text"], comment)
+        self.assertEquals(bug.sub_components, {"lvm2": [sub_component]})
 
         # Close the bug
         tests.clicomm("bugzilla modify --close WONTFIX %s" % bugid,
@@ -389,7 +400,7 @@ class RHPartnerTest(BaseTest):
 
         # modify --target_release
         # modify --target_milestone
-        targetbugid = 831888
+        targetbugid = 492463
         targetbug = bz.getbug(targetbugid)
         targetcmd = "bugzilla modify %s " % targetbugid
         tests.clicomm(targetcmd +
@@ -417,10 +428,10 @@ class RHPartnerTest(BaseTest):
         # modify --os
         # modify --platform
         # modify --version
-        tests.clicomm(cmd + "--version 18 --os Windows --arch ppc "
+        tests.clicomm(cmd + "--version rawhide --os Windows --arch ppc "
                             "--url http://example.com", bz)
         bug.refresh()
-        self.assertEquals(bug.version, "18")
+        self.assertEquals(bug.version, "rawhide")
         self.assertEquals(bug.op_sys, "Windows")
         self.assertEquals(bug.platform, "ppc")
         self.assertEquals(bug.url, "http://example.com")
@@ -507,8 +518,8 @@ class RHPartnerTest(BaseTest):
 
         self.assertEquals(len(out), 3)
         self.assertEquals(fname, "bz-attach-get1.txt")
-        self.assertEquals(file(fname).read(),
-                          file(testfile).read())
+        self.assertEquals(open(fname).read(),
+                          open(testfile).read())
         os.unlink(fname)
 
         # Get all attachments
@@ -696,10 +707,27 @@ class RHPartnerTest(BaseTest):
             bz.cookiefile = None
             raise AssertionError("Setting cookiefile for active connection "
                                  "should fail.")
-        except RuntimeError, e:
+        except RuntimeError:
+            e = sys.exc_info()[1]
             self.assertTrue("disconnect()" in str(e))
 
         bz.disconnect()
         bz.cookiefile = None
         bz.connect()
         self.assertFalse(bool(self._check_rh_privs(bz, "", True)))
+
+    def test13SubComponents(self):
+        bz = self.bzclass(url=self.url, cookiefile=cf)
+        # Long closed RHEL5 lvm2 bug. This component has sub_components
+        bug = bz.getbug("185526")
+        self.assertEquals(bug.component, "lvm2")
+
+        bz.update_bugs(bug.id, bz.build_update(
+            component="lvm2", sub_component="Command-line tools (RHEL5)"))
+        bug.refresh()
+        self.assertEqual(bug.sub_components,
+            {"lvm2": ["Command-line tools (RHEL5)"]})
+
+        bz.update_bugs(bug.id, bz.build_update(sub_component={}))
+        bug.refresh()
+        self.assertEqual(bug.sub_components, {})
